#ifndef HW_SPI_H
#define HW_SPI_H

/*
 * Spi - simple access to USCI SPI hardware
 */
#include <msp430.h>

enum SPI_PHASE {RISING_EDGE,FALLING_EDGE};

enum SPI_CLK_POLARITY {SPI_CLK_HIGH,SPI_CLK_LOW};

enum SPI_MSB {LSBFIRST, MSBFIRST};

unsigned char SPITXData;
unsigned char SPIRXData;

template<
volatile unsigned int &CTLW0,
volatile unsigned char &BR0,
volatile unsigned char &BR1,
volatile unsigned int &IE,
volatile unsigned char &SCLK_PSEL1,
unsigned char SCLK_BIT,
volatile unsigned char &MOSI_and_SOMI_PSEL1,
unsigned char MOSI_BIT,
unsigned char SOMI_BIT
>
class USCI_SPI_Template {
	public:
		/**
		 * setupinterface - setup the USCI SPI Hardware
		 */
		void setupinterface(SPI_PHASE phase = RISING_EDGE, SPI_CLK_POLARITY polarity = SPI_CLK_HIGH, SPI_MSB msb = LSBFIRST) {
			// Configure SPI pins
			SCLK_PSEL1 |= SCLK_BIT;
			MOSI_and_SOMI_PSEL1 |= MOSI_BIT + SOMI_BIT;

			// Configure the UART
			CTLW0 |= UCSWRST;				// **Put state machine in reset**
			CTLW0 |= UCMST+UCSYNC;   		// 3-pin, 8-bit SPI master
			if(polarity == SPI_CLK_HIGH)	{
				CTLW0 |= UCCKPL;
			}
			if(msb == MSBFIRST)	{
				CTLW0 |= UCMSB;
			}
			if(phase == FALLING_EDGE){
				CTLW0 |= UCCKPH;
			}
			CTLW0 |= UCSSEL__ACLK;          // ACLK
			BR0 = 0x80;                     // /2
			BR1 = 1;                        //
			//UCB0MCTLW = 0;                    // No modulation
			CTLW0 &= ~UCSWRST;              // **Initialize USCI state machine**
			//UCB0IE |= UCRXIE;                 // Enable USCI_A0 RX interrupt

			//UCB0IE &= ~(UCRXIE | UCTXIE);		// disable interrupts
			IE |= UCRXIE;
		}

		inline bool empty() {
			return true;
			//return _recv_buffer.empty();
		}

		inline unsigned int recv() {
			//return _recv_buffer.pop_front();
			return 0;
		}

		inline void xmitByte(unsigned char byte)	{
			SPITXData = byte;

			// GO TO SLEEP AND WAIT FOR INTERRUPTS TO FIRE
			IE |= UCTXIE;			// force interrupt to on,
			__bis_SR_register(LPM0_bits + GIE);      // CPU off, enable interrupts
			__no_operation();                       // Remain in LPM0

			// WHEN RESUME HERE, byte has been sent
			__delay_cycles(2000);                   // Delay before next transmission
		}

		void xmitMultipleBytes(const unsigned char *byte_ptr, const unsigned int number_of_bytes)	{
			for(int i = number_of_bytes; i>0; i--)	{
				xmitByte((unsigned char) *byte_ptr);
				++byte_ptr;
			}
		}

		inline void waitForTx(void)	{
			/*
			while (!(UCB0IFG & UCTXIFG))
				; // USCI_B0 TX buffer ready?
				*/
		}

	private:

		#pragma vector=USCI_B0_VECTOR
		static __interrupt void USCI_ISR(void)	{
			switch(__even_in_range(UCB0IV,0x04))
			{
				case 0: break;                          // Vector 0 - no interrupt
				case 2:
					P3OUT &= ~BIT2;
					SPIRXData = UCB0RXBUF;
					UCB0IFG &= ~UCRXIFG;

					__bic_SR_register_on_exit(CPUOFF);// Wake up to setup next TX
					break;
				case 4:
					UCB0TXBUF = SPITXData;             // Transmit characters
					UCB0IE &= ~UCTXIE;
					P3OUT |= BIT2;
					break;
				default: break;
			}
		}
};

typedef USCI_SPI_Template<UCB0CTLW0,UCB0BR0,UCB0BR1,UCB0IE,P2SEL1,BIT2,P1SEL1,BIT6,BIT7> SPI_USCI_B0;
static SPI_USCI_B0 SPI;



#endif /* HW_SERIAL_H */
